
/*
 *  Project     [tie], ontic
 *  Author      TriX
 *  Description TI92 linker for the parallel & serial cable
 *  Module      Main
 */

#include "ti92.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int SendHeader (TIHeader tih)
{
   if (!SendByte (0x9)) return 0;
   SendByte (tih.Type & 0xff);
   SendByte ((tih.Stuff >> 8) & 0xff);
   SendByte (tih.Stuff & 0xff);
   return 1;
} /* SendHeader */

int SendHInts (int type, int stuff)
{
   if (!SendByte (0x9)) return 0;
   SendByte (type & 0xff);
   SendByte ((stuff >> 8) & 0xff);
   SendByte (stuff & 0xff);
   return 1;
}

int GetChar ()
{
   unsigned char tmp;

   GetByte (&tmp);

   return tmp;
}


int GetWord ()
{
   unsigned char tmp[2];

   GetByte (&tmp[0]);
   GetByte (&tmp[1]);

   return (tmp[1] << 8) + (tmp[0]);   /* Conversion to the right format */
}


long GetDWord ()
{
   unsigned char tmp[4];

   GetByte (&tmp[0]);
   GetByte (&tmp[1]);
   GetByte (&tmp[2]);
   GetByte (&tmp[3]);

   return (tmp[3] << 24) + (tmp[2] << 16) + (tmp[1] << 8) + (tmp[0]);
}


int GetHeader (TIHeader *tih)
{
   unsigned char tmp;
   int k;

   if (!GetByte (&tmp)) return 0;
   k = tmp;
   if (k != 0x89) return 0;

   tih->Type = GetChar ();
   tih->Stuff = GetWord ();

   return 1;
} /* GetHeader */

int GetBody (char *savewhere, int *size)
{
   char  *myptr, tmp;
   int   i;

   TIHeader tih;

   if (!GetHeader (&tih))  return 0;
   if (tih.Type != 0x15)   return 0;

   GetChar (); GetChar (); GetChar (); GetChar ();

   myptr = savewhere;
   if (size != NULL) *size = tih.Stuff - 4;

   for (i = 0; i < tih.Stuff - 4; i++)
   {
      GetByte (myptr);
      myptr++;
   } /* for */

   GetByte (&tmp); GetByte (&tmp);          /* Ignore checksum */

   return 1;
} /* GetBody */

int SendVarHeader (VarHeader vh)
{
   int HSize = 6 + vh.Namelength, chksum, i;

   SendHInts (0xa2, ((HSize & 0xff) << 8) + ((HSize >> 8) & 0xff));
   chksum = 0;

   SendByte ((vh.VariableSize >> 0)  & 0xff);
   SendByte ((vh.VariableSize >> 8)  & 0xff);
   SendByte ((vh.VariableSize >> 16) & 0xff);
   SendByte ((vh.VariableSize >> 24) & 0xff);

   chksum += (vh.VariableSize >> 0)  & 0xff;
   chksum += (vh.VariableSize >> 8)  & 0xff;
   chksum += (vh.VariableSize >> 16) & 0xff;
   chksum += (vh.VariableSize >> 24) & 0xff;

   SendByte (vh.VariableType);
   SendByte (vh.Namelength);

   for (i = 0; i < vh.Namelength; i++)
   {
      SendByte (vh.VariableName[i]);
      chksum += vh.VariableName[i];
   }

   chksum += (vh.VariableType & 0xff);
   chksum += (vh.Namelength   & 0xff);

   /* Send chechsum */
   SendByte ((chksum >> 0) & 0xff);
   SendByte ((chksum >> 8) & 0xff);

   return 0;
}

int GetVarHeader (VarHeader *Savein)
{
   int i;

   Savein->VariableSize = GetDWord ();
   Savein->VariableType = GetChar ();
   Savein->Namelength   = GetChar ();

   for (i = 0; i < Savein->Namelength; i++)
      Savein->VariableName[i] = GetChar ();

   Savein->VariableName[i] = '\0';

   Savein->Checksum = GetWord ();

   return 1;
}



