
/*  Project     ontic
 *  Author      MXM-TriX
 *  Module      Conversion routine. This module convert PC source to TI
 *              source (in both directions)
 *  First       1999-11-23
 *  Last update 1999-11-23
 */

#include <stdio.h>
#include <stdlib.h>
#include <dos.h>
#include <string.h>
#include "ti92.h"

struct CV_Entry {
     char PC_Char;
     char TI_Char;
};

struct CV_Entry ConversionTable[4] = {{'', 228},
                                      {'', 252},
                                      {'', 248},
                                      {'', 129}};

int SizeOf_CVT = 4;

char ProjectName[256];          /* ontic project name = folder name */

int Send_Source (char *FileName)
{
   FILE         *input;
   char         *MemoryArea, File[80];
   long         SourceSize, i, j, FSize, temp, ChkSum;
   VarHeader    VH_TIXX;
   TIHeader     TH_TIXX;

   input = fopen (FileName, "r");

   /* Extract filename */
   i = 0;
   while (FileName[i] != '.')
   {
      File[i] = FileName[i];
      File[i + 1] = '\0';
      i++;
   }

   FSize = filelength (fileno (input)) + 1;
   MemoryArea = (char *) malloc (FSize);

   /* Load file into the memoryarea, calculate size and convert data */
   SourceSize = 1;
   MemoryArea[0] = ' ';

   for (i = 0; i < FSize; i++)
   {
      temp = fgetc (input);
      if (temp == '\r') continue;    /* Skip    '\r' */
      if (temp == '\t') temp = ' ';  /* Convert '\t' */
      if (temp == '\n')
      {
         MemoryArea[SourceSize] = 13;
         MemoryArea[SourceSize + 1] = ' ';
         SourceSize += 2;
      }

      /* Conversion table */
      for (j = 0; j < SizeOf_CVT; j++)
          if (temp == ConversionTable[j].PC_Char)
          {
             temp = ConversionTable[j].TI_Char;
             break;
          } /* if */

      /* Add this character */
      MemoryArea[SourceSize] = temp;
      SourceSize++;
   } /* for */

   /* Prepare link to the TI92 / TI89 */

   VH_TIXX.VariableSize = SourceSize + 6;
   VH_TIXX.VariableType = 0x0b;         /* 0x0b = Text */
   sprintf (VH_TIXX.VariableName, "%s\\%s", ProjectName, File);
   VH_TIXX.Namelength = strlen (VH_TIXX.VariableName);

   /*
   SendHInts (0x68, 0x0000);

   GetHeader (&TH_TIXX);


   if (TH_TIXX.Stuff != 0)
      return -1;
   */

   SendVarHeader (VH_TIXX);
   GetHeader (&TH_TIXX);

   if (TH_TIXX.Stuff != 0)
      return -1;

   GetHeader (&TH_TIXX);
   if (TH_TIXX.Type != 0x09)
      return -1;

   SendHInts (0x56, 0x0000);
   SendHInts (0x15, ((SourceSize & 0xff) << 8) + ((SourceSize >> 8) & 0xff));

   /*
   SendByte (0x00); SendByte (0x00);
   SendByte (((SourceSize + 4) >> 24) & 0xff);
   SendByte (((SourceSize + 4) >> 16) & 0xff);
   SendByte (((SourceSize + 4) >>  8) & 0xff);
   SendByte (((SourceSize + 4) >>  0) & 0xff);

   SendByte (0x00);
   SendByte (0x00);
   ChkSum = (((SourceSize + 4) >> 24) & 0xff) +
            (((SourceSize + 4) >> 16) & 0xff) +
            (((SourceSize + 4) >>  8) & 0xff) +
            (((SourceSize + 4) >>  0) & 0xff);
   */

   SendByte (0x00);
   SendByte (0x00);
   for (i = 0; i < SourceSize; i++)
   {
      ChkSum += MemoryArea[i];
      SendByte (MemoryArea[i]);
   }

   SendByte (0x00); SendByte (0xe0);
   ChkSum += 0xe0;

   SendByte ((ChkSum & 0xff));
   SendByte ((ChkSum >> 8) & 0xff);
   GetHeader (&TH_TIXX);

   if (TH_TIXX.Stuff != 0)
      return -1;

   SendHInts (0x92, 0x0000);
   GetHeader (&TH_TIXX);

   free (MemoryArea);

   return 1;
} /* Send_Source */

int Get_Source (char *FileName)
{

   return 1;
} /* Get_Source */

int main (int argc, char *argv[])
{
    printf ("ontic by MXM Projects\n\n");

    SetPort (LPT_PORT_1);
    if (argc != 4)
    {
       printf ("Help\n\n"
               "ontic  -[s|r] projectname files\n\n"
               "   -s  Send project\n"
               "   -r  Receive project\n\n"
               "The projectname has to be a folder (on PC or TI92, TI89)\n");
       return 0;
    } /* if */

    strcpy (ProjectName, argv[2]);
    strlwr (ProjectName);

    if (strcmp (argv[1], "-S") == 0 || strcmp (argv[1], "-s") == 0)
    {
       /* Send project */
       Send_Source (argv[3]);
    } else if (strcmp (argv[1], "-R") == 0 || strcmp (argv[1], "-r") == 0) {
       /* Receive project */
       Get_Source (argv[3]);
    } else
       printf ("Wrong options\n");

    return 0;
} /* main */
